@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,6 +19,10 @@
 **/
 
 
+/**
+ * @var CView $this
+ */
+
 if ($data['uncheck']) {
 	uncheckTableRows('proxy');
 }
@@ -61,7 +65,11 @@ $proxyTable = (new CTableInfo())
 			(new CCheckBox('all_hosts'))
 				->onClick("checkAll('".$proxyForm->getName()."', 'all_hosts', 'proxyids');")
 		))->addClass(ZBX_STYLE_CELL_WIDTH),
-		make_sorting_header(_('Name'), 'host', $data['sort'], $data['sortorder']),
+		make_sorting_header(_('Name'), 'host', $data['sort'], $data['sortorder'],
+			(new CUrl('zabbix.php'))
+				->setArgument('action', 'proxy.list')
+				->getUrl()
+		),
 		_('Mode'),
 		_('Encryption'),
 		_('Compression'),
@@ -98,7 +106,9 @@ foreach ($data['proxies'] as $proxy) {
 			$hosts[] = ', ';
 		}
 
-		$hosts[] = (new CLink($host['name'], 'hosts.php?form=update&hostid='.$host['hostid']))->addClass($style);
+		$hosts[] = $data['allowed_ui_conf_hosts']
+			? (new CLink($host['name'], 'hosts.php?form=update&hostid='.$host['hostid']))->addClass($style)
+			: (new CSpan($host['name']))->addClass($style);
 	}
 
 	$name = new CLink($proxy['host'], 'zabbix.php?action=proxy.edit&proxyid='.$proxy['proxyid']);
