@@ -25,6 +25,7 @@ const DASHBOARD_PAGE_STATE_DESTROYED = 'destroyed';
 
 const DASHBOARD_PAGE_EVENT_EDIT = 'dashboard-page-edit';
 const DASHBOARD_PAGE_EVENT_WIDGET_ADD = 'dashboard-page-widget-add';
+const DASHBOARD_PAGE_EVENT_WIDGET_ADD_NEW = 'dashboard-page-widget-add-new';
 const DASHBOARD_PAGE_EVENT_WIDGET_DELETE = 'dashboard-page-widget-delete';
 const DASHBOARD_PAGE_EVENT_WIDGET_POSITION = 'dashboard-page-widget-position';
 const DASHBOARD_PAGE_EVENT_WIDGET_EDIT = 'dashboard-page-widget-edit';
@@ -104,8 +105,9 @@ class CDashboardPage extends CBaseComponent {
 		if (this._is_edit_mode) {
 			this._initWidgetDragging();
 			this._initWidgetResizing();
-			this._initWidgetPlaceholder();
 		}
+
+		this._initWidgetPlaceholder();
 	}
 
 	// Logical state control methods.
@@ -125,13 +127,7 @@ class CDashboardPage extends CBaseComponent {
 	activate() {
 		this._state = DASHBOARD_PAGE_STATE_ACTIVE;
 
-		if (this._is_edit_mode) {
-			this._resizeGrid(this._getNumOccupiedRows() + this._grid_pad_rows);
-		}
-		else {
-			this._resizeGrid();
-		}
-
+		this._resizeGrid();
 		this._activateEvents();
 
 		for (const widget of this._widgets.keys()) {
@@ -142,8 +138,9 @@ class CDashboardPage extends CBaseComponent {
 		if (this._is_edit_mode) {
 			this._activateWidgetDragging();
 			this._activateWidgetResizing();
-			this.resetWidgetPlaceholder();
 		}
+
+		this.resetWidgetPlaceholder();
 	}
 
 	_activateWidget(widget) {
@@ -167,9 +164,9 @@ class CDashboardPage extends CBaseComponent {
 		}
 
 		this._deactivateEvents();
+		this._deactivateWidgetPlaceholder();
 
 		if (this._is_edit_mode) {
-			this._deactivateWidgetPlaceholder();
 			this._deactivateWidgetDragging();
 			this._deactivateWidgetResizing();
 		}
@@ -220,10 +217,9 @@ class CDashboardPage extends CBaseComponent {
 
 		this._initWidgetDragging();
 		this._initWidgetResizing();
-		this._initWidgetPlaceholder();
 
 		if (this._state === DASHBOARD_PAGE_STATE_ACTIVE) {
-			this._resizeGrid(this._getNumOccupiedRows() + this._grid_pad_rows);
+			this._resizeGrid();
 
 			this._activateWidgetDragging();
 			this._activateWidgetResizing();
@@ -704,18 +700,14 @@ class CDashboardPage extends CBaseComponent {
 	};
 
 	_resizeGrid(min_rows = null) {
-		if (min_rows === 0) {
-			this._grid_min_rows = null;
+		if (min_rows == 0) {
+			this._grid_min_rows = 0;
 		}
 		else if (min_rows !== null) {
 			this._grid_min_rows = Math.max(this._grid_min_rows, Math.min(this._max_rows, min_rows));
 		}
 
-		let num_rows = this._getNumOccupiedRows();
-
-		if (this._grid_min_rows !== null) {
-			num_rows = Math.max(num_rows, this._grid_min_rows);
-		}
+		let num_rows = Math.max(this._grid_min_rows, this._getNumOccupiedRows());
 
 		let height = this._cell_height * num_rows;
 
@@ -783,6 +775,7 @@ class CDashboardPage extends CBaseComponent {
 		this._widget_placeholder_pos = null;
 		this._widget_placeholder_clicked_pos = null;
 		this._widget_placeholder_is_active = false;
+		this._widget_placeholder_is_edit_mode = null;
 		this._widget_placeholder_move_animation_frame = null;
 
 		this._dashboard_grid.appendChild(this._widget_placeholder.getNode());
@@ -938,6 +931,15 @@ class CDashboardPage extends CBaseComponent {
 		};
 
 		this._widget_placeholder_events = {
+			addNewWidget: () => {
+				if (!this._is_edit_mode) {
+					this.setEditMode();
+					this.fire(DASHBOARD_PAGE_EVENT_EDIT);
+				}
+
+				this.fire(DASHBOARD_PAGE_EVENT_WIDGET_ADD_NEW);
+			},
+
 			mouseDown: (e) => {
 				if (e.button != 0) {
 					return;
@@ -992,7 +994,12 @@ class CDashboardPage extends CBaseComponent {
 				});
 			},
 
-			mouseLeave: (e) => {
+			mouseLeave: () => {
+				if (this._widget_placeholder_move_animation_frame !== null) {
+					cancelAnimationFrame(this._widget_placeholder_move_animation_frame);
+					this._widget_placeholder_move_animation_frame = null;
+				}
+
 				if (this._widget_placeholder_clicked_pos === null) {
 					this.resetWidgetPlaceholder();
 				}
@@ -1007,6 +1014,10 @@ class CDashboardPage extends CBaseComponent {
 	}
 
 	resetWidgetPlaceholder() {
+		if (this._widget_placeholder_is_active && this._widget_placeholder_is_edit_mode != this._is_edit_mode) {
+			this._deactivateWidgetPlaceholder();
+		}
+
 		if (!this._widget_placeholder_is_active) {
 			this._activateWidgetPlaceholder();
 		}
@@ -1014,55 +1025,62 @@ class CDashboardPage extends CBaseComponent {
 		this._widget_placeholder_pos = null;
 		this._widget_placeholder_clicked_pos = null;
 
-		if (this._is_editable) {
-			if (this._is_kiosk_mode) {
-				this._widget_placeholder.setState(WIDGET_PLACEHOLDER_STATE_KIOSK_MODE);
-			}
-			else {
-				this._widget_placeholder.setState(WIDGET_PLACEHOLDER_STATE_ADD_NEW);
-			}
+		if (this._is_editable && this._widgets.size == 0) {
+			this._widget_placeholder
+				.setState(WIDGET_PLACEHOLDER_STATE_ADD_NEW)
+				.showAtDefaultPosition();
 		}
 		else {
-			this._widget_placeholder.setState(WIDGET_PLACEHOLDER_STATE_READONLY);
-		}
-
-		if (this._widgets.size == 0) {
-			this._widget_placeholder.showAtDefaultPosition();
+			this._widget_placeholder.hide();
 		}
 	}
 
 	_activateWidgetPlaceholder() {
-		this._widget_placeholder.on('mousedown', this._widget_placeholder_events.mouseDown);
+		this._widget_placeholder.on(WIDGET_PLACEHOLDER_EVENT_ADD_NEW_WIDGET,
+			this._widget_placeholder_events.addNewWidget
+		);
 
-		this._dashboard_grid.addEventListener('mousemove', this._widget_placeholder_events.mouseMove);
-		this._dashboard_grid.addEventListener('mouseleave', this._widget_placeholder_events.mouseLeave);
+		if (this._is_edit_mode) {
+			this._widget_placeholder.on('mousedown', this._widget_placeholder_events.mouseDown);
 
-		document.querySelector('.wrapper').addEventListener('scroll', this._widget_placeholder_events.scroll);
+			this._dashboard_grid.addEventListener('mousemove', this._widget_placeholder_events.mouseMove);
+			this._dashboard_grid.addEventListener('mouseleave', this._widget_placeholder_events.mouseLeave);
+
+			document.querySelector('.wrapper').addEventListener('scroll', this._widget_placeholder_events.scroll);
+		}
 
 		this._widget_placeholder_is_active = true;
+		this._widget_placeholder_is_edit_mode = this._is_edit_mode;
 	}
 
 	_deactivateWidgetPlaceholder({do_hide = true} = {}) {
-		if (this._widget_placeholder_move_animation_frame !== null) {
-			cancelAnimationFrame(this._widget_placeholder_move_animation_frame);
-			this._widget_placeholder_move_animation_frame = null;
+		this._widget_placeholder.off(WIDGET_PLACEHOLDER_EVENT_ADD_NEW_WIDGET,
+			this._widget_placeholder_events.addNewWidget
+		);
+
+		if (this._widget_placeholder_is_edit_mode) {
+			if (this._widget_placeholder_move_animation_frame !== null) {
+				cancelAnimationFrame(this._widget_placeholder_move_animation_frame);
+				this._widget_placeholder_move_animation_frame = null;
+			}
+
+			this._widget_placeholder.off('mousedown', this._widget_placeholder_events.mouseDown);
+
+			this._dashboard_grid.removeEventListener('mousemove', this._widget_placeholder_events.mouseMove);
+			this._dashboard_grid.removeEventListener('mouseleave', this._widget_placeholder_events.mouseLeave);
+
+			document.querySelector('.wrapper').removeEventListener('scroll', this._widget_placeholder_events.scroll);
+
+			document.removeEventListener('mousemove', this._widget_placeholder_events.mouseMove);
+			document.removeEventListener('mouseup', this._widget_placeholder_events.mouseUp);
 		}
 
-		this._widget_placeholder.off('mousedown', this._widget_placeholder_events.mouseDown);
-
-		this._dashboard_grid.removeEventListener('mousemove', this._widget_placeholder_events.mouseMove);
-		this._dashboard_grid.removeEventListener('mouseleave', this._widget_placeholder_events.mouseLeave);
-
-		document.querySelector('.wrapper').removeEventListener('scroll', this._widget_placeholder_events.scroll);
-
-		document.removeEventListener('mousemove', this._widget_placeholder_events.mouseMove);
-		document.removeEventListener('mouseup', this._widget_placeholder_events.mouseUp);
-
 		if (do_hide) {
 			this._widget_placeholder.hide();
 		}
 
 		this._widget_placeholder_is_active = false;
+		this._widget_placeholder_is_edit_mode = null;
 	}
 
 	// Widget dragging methods.
@@ -1909,12 +1927,16 @@ class CDashboardPage extends CBaseComponent {
 			},
 
 			widgetEnter: (e) => {
+				const widget = e.detail.target;
+
 				if (this._is_edit_mode) {
+					const pos = widget.getPos();
+
+					this._resizeGrid(pos.y + pos.height + this._grid_pad_rows);
+
 					this.resetWidgetPlaceholder();
 				}
 
-				const widget = e.detail.target;
-
 				if (!widget.isEntered()) {
 					widget.enter();
 					this._leaveWidgets({except_widget: widget});
